
package graphComponents;

import graphComponents.unionfind.UnionFind; 
import graphhandler.FileChooserFilter;
import graphhandler.VGraph;
import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

/** Graph class represents a graph by using a Neighbouring List to represent
 * the Vertexs and their Edges
 */
public class Graph{

    /** HashMaps that contain the vertexs, their adjacency list and a list with those that where visited during an algorithm or not */
    private HashMap<Integer, Vertex> vertexs;
    private HashMap<Integer, LinkedList<Edge>> adjacencyMap;
    private HashMap<Integer, Boolean> visited;

    /** The graphs scc*/
    private GraphSCC scc;
    /** Counter used to set the pre/post values during the algorithm runs*/
    private int clock;   
    private boolean foundPath;
    private boolean failed;
    private File sourceFile;
    private VGraph vGraph;
    private JScrollPane panel;
    
    /**Panel refreshing Constants*/
    public final static int SHOW_INITIAL_GRAPH=0;
    public final static int SHOW_SCC=1;
    public final static int SHOW_KRUSKAL=2;
    private final static int SHOW_LAST=2;

    /** Empty constructor. */
    public Graph() {
        vertexs = new HashMap<Integer, Vertex>(30);
        adjacencyMap = new HashMap<Integer, LinkedList<Edge>>(30);
        visited = new HashMap<Integer, Boolean>(30);
        scc= new GraphSCC();
        clock =0;
        failed=false;
    }

    /** Constructor that reads the edges of the graph from the source file
     * @param source is the file from which it will attempt to read
     * @throws FileNotFoundException  
    */
    public Graph(File source) throws FileNotFoundException{
        this();
        if (source != null && source.exists() ){
             Scanner in = new Scanner(new BufferedReader(new InputStreamReader(new FileInputStream(source))));
             StringTokenizer token=new StringTokenizer("");
             setSourceFile(source);
             String temp1, temp2, temp3;
             
             try {
                while(in.hasNext()){                //while it hasn't reached the end of the stream
                    in.useDelimiter("\t");
                    token= new StringTokenizer( in.next() );
                    temp1= token.nextToken().replace('\r', ' ').replace('\n', ' ').trim() ;
                    in.useDelimiter("\t");
                    temp2= in.next();
                    in.reset();                     //resets the delimiter to the default
                    temp3= in.next(); 
                    addEdge(Integer.parseInt( temp1), Integer.parseInt(temp2), Float.parseFloat(temp3));
                }
             } catch(Exception e) {
                failed=true;
                JOptionPane.showMessageDialog(new JPanel(), "Error in reading file" + source.getName() +".\n Incompatible format.", "Error", JOptionPane.ERROR_MESSAGE);
             }
             in.close();
        }else{
            failed = true;
            JOptionPane.showMessageDialog(new JPanel(), "Error, file does not exist.", "Error", JOptionPane.ERROR_MESSAGE);
        }
    } 

    /**Add's a new Vertex with id to the HashMaps
     * @param id
     * @return  
     */
    public boolean addVertex(int id){
        if(containsVertex(id))
            return false;
        vertexs.put(id , new Vertex(id));
        adjacencyMap.put(id, new LinkedList());
        return true;
    }

    /**Add's the v Vertex to the HashMaps
     * @param v 
     * @return 
     */
    public boolean addVertex(Vertex v){
        return addVertex(v.getId());
    }

    /** Add's a new edge in the Graph
     * @param e
     * @return  
     */
    public boolean addEdge(Edge e){
        return addEdge(e.getFrom(), e.getTo(), e.getWeight());
    }

    /**Add's a new edge in the Graph
     * @param v1 
     * @param v2
     * @param weight
     * @return  
     */
    public boolean addEdge(Vertex v1, Vertex v2, float weight){
        return addEdge(v1.getId(), v2.getId(), weight);
    }

    /**Add's a new edge in the Graph
     * @param id1
     * @param id2
     * @param weight 
     * @return  
     */
    public boolean addEdge(int id1, int id2, float weight){
        //if(containsEdge(id1,id2) )
          //  return false;
        addVertex(id1);
        addVertex(id2);
        LinkedList list=adjacencyMap.get(id1);
        list.add(new Edge(weight, id1, id2 ));
        list=null; 
        return true;
    }

    /**Remove's an edge from the Graph
     * @param id1
     * @param id2
     * @return  
     */
    public boolean removeEdge(int id1, int id2){
        if(containsEdge(id1,id2) ){
            LinkedList list=(LinkedList) adjacencyMap.get(id1);
            int tempid;
            for(int i=0;list!=null &&  i< list.size(); i++){
                    tempid =((Edge) list.get(i)).getTo() ;
                    if(tempid==id2){
                        list.remove((Object) ((Edge) list.get(i)));
                        return true;
                    }
            }
        }
        return false;
    }

    /**Remove's an edge from the Graph
     * @param v1
     * @param v2 
     * @return  
     */
    public boolean removeEdge(Vertex v1, Vertex v2){
        return removeEdge(v1.getId(), v2.getId());
    }

    /**Removes a vertex and all it's edges from the graph
     * @param id
     * @return  
     */
    public boolean removeVertex(int id){
        if(!containsVertex(id))
            return false;
        vertexs.remove(id);                     //remove the vertex
        adjacencyMap.remove(id);                //remove the edges that go from the vertex
        Set keys=adjacencyMap.keySet();         //and then remove the edges that went to the vertex
        for (Iterator iter = keys.iterator(); iter.hasNext();)
            if(! removeEdge((Integer) iter.next(), id))
                return false;
        return true;
    }

    /**Removes a vertex v and all it's edges from the graph
     * @param v
     * @return  
     */
    public boolean removeVertex(Vertex v){
        return removeVertex(v.getId());
    }

    /**Returns true if the edge between id1 and id2 already exists.
     * @param id1 
     * @param id2
     * @return  
     */
    public boolean containsEdge(int id1, int id2){
        LinkedList list=(LinkedList) adjacencyMap.get(id1);
        int tempid;
        for(int i=0;list!=null &&  i< list.size(); i++){
                tempid =((Edge) list.get(i)).getTo() ;
                if(tempid==id2)
                    return true;
        }
        list=null; System.gc();
        return false;
    }

    /**Returns true if the edge between v1 and v2 already exists.
     * @param v1
     * @param v2 
     * @return  
     */
    public boolean containsEdge(Vertex v1, Vertex v2){
        return containsEdge(v1.getId(), v2.getId());
    }

    /**Return the number of the Vertexs added to the Graph.
     * @return 
     */
    public int getNodesNum() {
        return vertexs.size();
    }

    /**Returns the number of edges in the Graph.
     * @return 
     */
    public int getEdgeCount() {
        int count = 0;
        Set s = vertexs.keySet(); LinkedList keys;
        for (Iterator iter = s.iterator(); iter.hasNext();) {
            keys = adjacencyMap.get((Integer) iter.next() );
            count += keys.size();
        }
        return count;
    }

    /** Sets the sourceFile of the Graph based on the parameter.
     * @param sourceFile 
     */
    public void setSourceFile(File sourceFile){
        if(sourceFile.exists())
            this.sourceFile=sourceFile;
        else this.sourceFile=null;
    }

    /** Sets the scc of the Graph based on the parameter.
     * @param scc 
     */
    public void setSCC(GraphSCC scc){
        this.scc=scc;
    }

    /**Returns true if Vertex v exists in Graph.
     * @param v
     * @return  
     */
    public boolean containsVertex(Vertex v) {
        return vertexs.containsKey(v.getId());
    }

    /**Returns true if Vertex's id  exists in Graph.
     * @param id
     * @return  
     */
    public boolean containsVertex(int id) {
        return vertexs.containsKey(id);
    }

    /**Returns the Transpose Graph of the initial Graph.
     * @return 
     */
    public Graph getTranspose(){
         Graph temp=new Graph();
         Set keys= vertexs.keySet();
         int id1=0,id2=0;
         LinkedList list;
         //add the existing edges reversed to the new Graph
         for(Iterator iter= keys.iterator() ; iter.hasNext(); ){
            id1 =(Integer) iter.next();
            list=(LinkedList) adjacencyMap.get(id1);
            for(int i=0;list!=null &&  i< list.size(); i++){
                id2 =((Edge) list.get(i)).getTo() ;
                temp.addEdge(id2, id1,((Edge) list.get(i)).getWeight() );
            }
         }
         keys=null; list=null; System.gc();
         return temp;
    }

    /**Explore the (sub)graph starting from v. */
    private void explore(Vertex v) {
        previsit(v);
        //for each edge (v, w) out of v do
        Vertex u;
        LinkedList list = adjacencyMap.get( v.getId() );
        for (int i = 0; i < list.size(); i++) {
            u = vertexs.get(((Edge) list.get(i)).getTo() );
            if (! visited.get(u.getId() ) ) {
                explore(u);
            }
        }
        list=null; System.gc();
        postvisit(v);
    }

    /**Same as explore with some changes to fit to SCC finding. */
    private Graph exploreSCC(Vertex v, Graph gr) {
        gr.addVertex(v);
        previsit(v);
        //for each edge (v, w) out of v do
        LinkedList edges = adjacencyMap.get( v.getId() );
        for (int i = 0; i < edges.size(); i++) {
            int idu = ((Edge) edges.get(i)).getTo() ;
            Vertex u = vertexs.get(idu);
            if (!visited.get(idu)) {
                gr.addEdge(v, u, ((Edge) edges.get(i)).getWeight());
                exploreSCC(u, gr);
            } else if (gr.containsVertex(u)) 
                gr.addEdge(v, u, ((Edge) edges.get(i)).getWeight());
        }
        postvisit(v);
        return gr;
    }
    
    /**Previsits a v vertex. */
    private void previsit(Vertex v){
        v.setPre(clock++);
        visited.put(v.getId(), Boolean.TRUE);
    }

    /**Postvisits a v vertex. */
    private void postvisit(Vertex v){
        v.setPost(clock++);
    }

    /**Performs a Depth-First Search to the Graph. */
    public void DFS(){
        clock=1; int id;
        Vertex v;
        Set keys= vertexs.keySet();
        //for each v in G visited = false
        for(Iterator iter= keys.iterator() ; iter.hasNext(); ){
            id =(Integer) iter.next();
            visited.put(id, Boolean.FALSE);
        }
        //for each v in G if not visited explore(v)
         for(Iterator iter= keys.iterator() ; iter.hasNext();  ){
            id =(Integer) iter.next();
            v=vertexs.get(id);
            if(! visited.get(id ))
                explore(v);
         }

    }

    /**Return an int ArrayList with the id of the Graph's Vertexes sorted by post. */
    protected ArrayList sortByPost(){
        ArrayList list= new ArrayList();
        Set s = vertexs.keySet();
        for (Iterator iter = s.iterator(); iter.hasNext();) {
            list.add( vertexs.get((Integer) iter.next()) );
        }
        Collections.sort(list);
        return list;
    }

    /**Finds the scc in the Graph.
     * @return 
     */
    public ArrayList <Graph> findSCC(){
        return scc.findSCC(this); 
    }

    /** Returns the GraphSCC object containing the graph's SCC.
     * @return 
     */
    public GraphSCC getSCC(){ return scc;}

    /**Does a DFS based on the order supplied in the ArrayList
     * @param DFSOrder 
     * @return returns a vector with the scc of the graph
     */
    public ArrayList<Graph> orderedDFS(ArrayList DFSOrder){
        Vertex v;
        for (int i = 0; i < DFSOrder.size(); i++)
             visited.put( ((Vertex) DFSOrder.get(i)).getId() , Boolean.FALSE);
        ArrayList<Graph> gSCC=new ArrayList<Graph>();
        for (int i = 0; i < DFSOrder.size(); i++) {
            v = (Vertex) DFSOrder.get(i);
            if (! visited.get(v.getId() ) ) 
                gSCC.add( exploreSCC(v, new Graph() ));
        }
        return gSCC;
    }

    /**BFS algorithm for the graph 
     * @param v 
     * @return returns a hashMap with the distances to the v vertex
     */
    public HashMap <Integer, Integer> BFS(Vertex v){
        HashMap<Integer, Integer> distances = new HashMap<Integer, Integer>(30);
        LinkedList<Integer> queue = new LinkedList<Integer>();
        distances.put(v.getId(), 0);
        queue.addLast(v.getId());
        while (!queue.isEmpty()) {
            int vid = queue.removeLast();
            LinkedList edges = adjacencyMap.get(vid);
            for (int i = 0; i < edges.size(); i++) {
                int id = ((Edge) edges.get(i)).getTo() ;
                if (!distances.containsKey(id)) {
                    distances.put(id, distances.get(vid) + 1);
                    queue.addLast(id);
                }
            }
        }
        return distances;
    }

    /**Gets the diameter of the Graph
     * @return 
     */
    public int getDiameter(){
        int diam=0;
        Set verts = vertexs.keySet();
        Vertex v = null;
        for (Iterator i = verts.iterator(); i.hasNext();) {
            int id = (Integer) i.next();
            v = vertexs.get(id);
            HashMap<Integer, Integer> dist = BFS(v);             //bfs gia ton ekastote kombo ston opoion eimai
            int maxDistance=0;
            Set dists = dist.keySet();                           //pairno ta distances pou epistrefei o bfs
            for (Iterator j = dists.iterator(); j.hasNext();) {  //kai brisko to megalutero
                int idd = (Integer) j.next();
                if (dist.get(idd) > diam){
                    diam = dist.get(idd);
                }
            }
            if(maxDistance > diam)
                diam= maxDistance;
        }
        return diam;
    }

    /**Returns a vector with the Diameter's Nodes
     * @return 
     */
    public Graph getPath(Vertex v1, Vertex v2, int pathSize, Graph gr){
        int id;
        foundPath=false;
        Set keys= vertexs.keySet();
        //for each v in G visited = false
        for(Iterator iter= keys.iterator() ; iter.hasNext(); ){
            id =(Integer) iter.next();
            visited.put(id, Boolean.FALSE);
        }
        explorePath(v1, v2, pathSize, gr);
        return gr;
    }
    
    /**Explores and returns a path as a Graph*/
    private Graph explorePath(Vertex v1, Vertex v2, int pathSize, Graph gr){
        gr.addVertex(v1);
        visited.put(v1.getId(), Boolean.TRUE);
        //for each edge (v, w) out of v do
        LinkedList edges = adjacencyMap.get( v1.getId() );
        for (int i = 0; i < edges.size(); i++) {
            int idu = ((Edge) edges.get(i)).getTo() ;
            Vertex u = vertexs.get(idu);
            if (!visited.get(idu)) {                                            //if the node hasn't been visited
                if(gr.getNodesNum()==pathSize && u.getId()==v2.getId()){     //if it is final node
                    gr.addEdge(v1, u, ((Edge) edges.get(i)).getWeight());
                    foundPath=true;
                    return gr;                                                  //the path is found
                }   
                else if(gr.getNodesNum()!=pathSize && foundPath){            //else if it was found remove the unnessesary edges
                    gr.removeEdge(v1, u);
                    visited.put(u.getId(), Boolean.FALSE);
                }else {
                    gr.addEdge(v1, u, ((Edge) edges.get(i)).getWeight());
                    explorePath(u, v2, pathSize, gr);
                }
            }
        }
        return gr;
    }

    /**Prints the graph 
     * @param stream is the BufferedWriter stream to which it will print the graph
     */
    public void print(BufferedWriter stream) {
        String output= toString();
        try {
            stream.write(output, 0, output.length());
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(new JFrame(), "Error, in printing Graph "+ sourceFile.getName() + ".", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    /**Saves the Graph to a file and returns true if everything goes well
     *@param target is the target file where the graph will be saved
     *@param type describes the extension of the file that the graph will be saved as("txt", "jpg", "png" etc)
     * @return  
     */
    public boolean saveAs(File target, String type){
        if(type.equalsIgnoreCase("txt")){
                try {
                    if(target==null) target=sourceFile;
                    if (!target.exists())
                        target.createNewFile();
                    BufferedWriter out = new BufferedWriter(new FileWriter(target));
                    Set s = vertexs.keySet();
                    if (s != null) {            //if there are any nodes
                        LinkedList edges;
                        Edge e;
                        for (Iterator iter = s.iterator(); iter.hasNext();) {                                   //for each node
                            int id = (Integer) iter.next();
                            edges = (LinkedList) adjacencyMap.get(id);
                            for (int j = 0; j < edges.size(); j++) {                                            //and for each node's edge
                                e = (Edge) edges.get(j) ;
                                out.write( vertexs.get(id) + "\t" + e.getTo()  +"\t"+ e.getWeight() +"\n");      //write it
                            }
                        }
                    }
                    out.flush();
                    out.close();
                } catch (Exception e) {
                    return false;
                }
        }else{
            vGraph.exportImage(target,  FileChooserFilter.getExtension(target) );
        }
        return true;
    }

    /** Updates the Graph's representation.*/
    public void updateVisualGraph(){
        vGraph.updateVGraph(panel.getWidth(), panel.getHeight());
    }

    /**Returns the visual representation of the graph in a JScrollpane.
     * @param displayGraph 
     * @return 
     */
    public JScrollPane getVisualGraph(int displayGraph){
        if(displayGraph==SHOW_INITIAL_GRAPH)
            vGraph = new VGraph(this);
        else if(displayGraph==SHOW_SCC){
            findSCC();
            vGraph = new VGraph(this, scc);
        } else if (displayGraph == SHOW_KRUSKAL)
            vGraph = new VGraph(this);
        else if(displayGraph>SHOW_LAST && displayGraph-(SHOW_LAST+1)<scc.size())
            vGraph = new VGraph(scc.get(displayGraph-(SHOW_LAST+1)));
        panel = new JScrollPane(vGraph.getVGraph(), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED); //graph
        return panel;
    }

    /**Returns a JPanel containing both the Side Panel and the visual representation of the Graph.
     * @param displayGraph 
     * @return 
     */
    public JPanel getGraphPanel(int displayGraph){
        JPanel gPanel=new JPanel(new BorderLayout());
        gPanel.setName(sourceFile.getName());
        gPanel.setBackground(Color.WHITE);
        //Load Graph representation
        gPanel.add(getVisualGraph(displayGraph), BorderLayout.CENTER);
        gPanel.add(getSidePanel(), BorderLayout.WEST);
        return gPanel;
    }

    /**Returns the side panel for the graph. */
    private JPanel getSidePanel(){
        JPanel sPanel=new JPanel(new GridBagLayout());
        sPanel.setBackground(Color.WHITE);
        GridBagConstraints cons=new GridBagConstraints();
        
        //Load Edges TextField
        JTextField edges= new JTextField("Edges:", 10);
        edges.setEditable(false);
        edges.setBackground(Color.WHITE);
        cons.fill = GridBagConstraints.BOTH;
        cons.gridx =0; cons.gridy = 0;
        cons.gridheight=1;cons.gridwidth=1;
        sPanel.add(edges, cons);

        //Load Graph's Edges
        JTextArea info= new JTextArea(toString(), 9, 10);
        info.setLineWrap(true);
        info.setEditable(false);
        info.setWrapStyleWord(true);
        cons.gridx =0; cons.gridy = 1;
        cons.gridheight=5;cons.gridwidth=1;
        sPanel.add(new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), cons);

        //Load MST Weight TextField
        JTextField mstWeightText= new JTextField("Graph's Weight:", 10);
        mstWeightText.setEditable(false);
        mstWeightText.setBackground(Color.WHITE);
        cons.gridx =0; cons.gridy = 6;
        cons.gridheight=1;cons.gridwidth=1;
        sPanel.add(mstWeightText, cons);

        //Load MST Weight TextField value
        JTextField mstWeightValue= new JTextField(""+ getGraphsWeight() , 10);
        mstWeightValue.setEditable(false);
        mstWeightValue.setBackground(Color.WHITE);
        cons.gridx =0; cons.gridy = 7;
        cons.gridheight=1;cons.gridwidth=1;
        sPanel.add(mstWeightValue, cons);

        //Load SCC # TextField
        JTextField sccNumText= new JTextField("Number of SCC:", 10);
        sccNumText.setEditable(false);
        sccNumText.setBackground(Color.WHITE);
        cons.gridx =0; cons.gridy = 8;
        cons.gridheight=1;cons.gridwidth=1;
        sPanel.add(sccNumText, cons);

        //Load SCC # TextField value
        JTextField sccNumValue= new JTextField(""+ scc.size() , 10);
        sccNumValue.setEditable(false);
        sccNumValue.setBackground(Color.WHITE);
        cons.gridx =0; cons.gridy = 9;
        cons.gridheight=1;cons.gridwidth=1;
        sPanel.add(sccNumValue, cons);

        //Load SCC TextField
        JTextField sccText= new JTextField("SCC's:", 10);
        sccText.setEditable(false);
        sccText.setBackground(Color.WHITE);
        cons.gridx =0; cons.gridy = 10;
        cons.gridheight=1;cons.gridwidth=1;
        sPanel.add(sccText, cons);

        //Load SCC info to TextArea
        JTextArea sccInfo= new JTextArea(SCCToString(), 13, 10);
        sccInfo.setLineWrap(true);
        sccInfo.setEditable(false);
        sccInfo.setWrapStyleWord(true);
        cons.gridx =0; cons.gridy = 11;
        cons.gridheight=6;cons.gridwidth=1;
        sPanel.add(new JScrollPane(sccInfo, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), cons);

        //Load TextField for Selecting each scc
        JTextField choiceText= new JTextField("Largest SCC:", 10);
        choiceText.setEditable(false);
        choiceText.setBackground(Color.WHITE);
        cons.gridx =0; cons.gridy = 17;
        cons.gridheight=1;cons.gridwidth=1;
        sPanel.add(choiceText, cons);
        
        //Load TextArea for displaying the largest SCC
        Graph lGraph=scc.getLargestSCC();
        JTextArea largestSCC= new JTextArea(" ", 6, 10);
        if(lGraph!=null)
           largestSCC.setText("Nodes: "+ lGraph.nodesToString()+ "\nSize:"+lGraph.getNodesNum()+"\nDiameter: "+ lGraph.getDiameter() );
        largestSCC.setLineWrap(true);
        largestSCC.setEditable(false);
        largestSCC.setWrapStyleWord(true);
        cons.gridx =0; cons.gridy = 18;
        cons.gridheight=2;cons.gridwidth=1;
        sPanel.add(new JScrollPane(largestSCC, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), cons);

        return sPanel;
    }

    /** Returns a boolean that is true if the reading of the graph's source file has failed.
     * @return 
     */
    public boolean readFailed(){ return failed;}

    /** Returns the file from which the graph was read/inputed.
     * @return 
     */
    public File getOriginFile(){ return sourceFile;}

    /** Compares the Graph with another Graph based on their origin file.
     * @param obj 
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null)  // Null references are not equal to this instance.
            return false;
        if (!obj.getClass ().equals (Edge.class)) //Neither are they instances of different classes
            return false;
        return (((Graph) obj).getOriginFile().getName().equalsIgnoreCase(sourceFile.getName()) );
    }

    /**Returns the graph's Edges in a string */
    @Override
    public String toString() {
        String gText="";
        Set s = vertexs.keySet();
        if (s != null) {
            LinkedList edges;
            for (Iterator iter = s.iterator(); iter.hasNext();) {
                int id = (Integer) iter.next();
                edges = (LinkedList) adjacencyMap.get(id);
                for (int j = 0; j < edges.size(); j++) 
                    gText += ((Edge) edges.get(j)).toString() + "\n";
            }
            if(!gText.equals(""))
                  return gText.substring(0, gText.length()-1);
        }
        return gText;
    }

    /**Returns the graph's Nodes in a string
     * @return 
     */
    public String nodesToString() {
        String gText="";
        Set s = vertexs.keySet();
        if (s != null) {
            for (Iterator iter = s.iterator(); iter.hasNext();) {
                gText += (Integer) iter.next() + ", ";
            }
        }
        return gText.substring(0, gText.length()-2);
    }

    /**Returns a String describing the edges of every SCC in the vector
     * @return 
     */
    public String SCCToString(){
        String gText="";
        for(int i=0; i<scc.size(); i++){
            gText +="SCC no."+ (i+1) + "\n" +"Size:"+ scc.get(i).getNodesNum(); 
            String edges=scc.get(i).toString();
            if(!edges.equalsIgnoreCase(""))
                gText +="\n\nEdges:\n" + scc.get(i).toString();
            gText+="\n--------\n";
        }

        return gText;
    }

    /** Returns a HashMap that contains the Graph's id's/Vertexs
     * @return 
     */
    public HashMap<Integer, Vertex> getVertexs(){ return vertexs; }

    /** Returns a HashMap that contains the Graph's vertex id's/LinkedLists with each vertex's outgoing edges
     * @return 
     */
    public HashMap<Integer, LinkedList<Edge>> getAdjacencyMap(){ return adjacencyMap; }

    /** Returns a minimum spanning tree (MST) by using the Kruskal algorithm
     * @return 
     */
    public Graph Kruskal(){
        Graph MST = new Graph();
        MST.setSourceFile(sourceFile);
        MST.setSCC(scc);
        UnionFind UF= new UnionFind("MST");
        //Ε: ταξινόμηση των ακμών κατά βάρος;
        Set keys= vertexs.keySet();
        LinkedList list;
        ArrayList <Edge> edges=new ArrayList<Edge>();
        for(Iterator iter= keys.iterator() ; iter.hasNext(); ){
            list=(LinkedList) adjacencyMap.get((Integer) iter.next());
            for(int i=0;list!=null &&  i< list.size(); i++)
                edges.add((Edge) list.get(i)) ;
         }
        Comparator comparator = Collections.reverseOrder();
        Collections.sort(edges,comparator);
        
        //for each v in V do MAKESET(v);
        Set s = vertexs.keySet();
        if (s != null) {
            for (Iterator iter = s.iterator(); iter.hasNext();) 
                UF.makeSet((Integer) iter.next());
        }
        //Find MST
        for(int i=0; i< edges.size(); i++){
            Edge next=edges.get(i);
            if(UF.find(next.getFrom() )!= UF.find(next.getTo() )){
                MST.addEdge(next);
                UF.union(next.getFrom() , next.getTo() );
            }
        }
        edges=null; System.gc();
        return MST;
    }

    /** Returns the total weight of all the graph's edges
     * @return 
     */
    public double getGraphsWeight(){
        double total=0d;
        Set keys= vertexs.keySet();
        LinkedList list;
        for(Iterator iter= keys.iterator() ; iter.hasNext(); ){
            list=(LinkedList) adjacencyMap.get((Integer) iter.next());
            for(int i=0;list!=null &&  i< list.size(); i++)
                total += ((Edge) list.get(i)).getWeight() ;
         }
        return total;
    }
    
}