
package graphComponents.unionfind;

import java.util.*;

/**Class that represents a UnionFind data type.
 */
public class UnionFind {

    String uf_id;
    ArrayList <UNode> nodes;

    /** Default constructor.*/
    public UnionFind(){
        nodes= new ArrayList <UNode>();
        this.uf_id="Union Find";
    }
    /* Constructor that also gives a name to the Union FInd object*/
    public UnionFind(String uf_id){
        nodes= new ArrayList <UNode>();
        this.uf_id=uf_id;
    }

    /** Constructor that creates a Union Find object with the initial node "firstElement".
     * @param firstElement 
     */
    public UnionFind(int firstElement){
        this();
        nodes.add(new UNode(firstElement));
    }

    /** Constructor that creates a Union Find object with the initial node "firstElement" and a "uf_if" name.
     * @param uf_id
     * @param firstElement  
     */
    public UnionFind(String uf_id, int firstElement){
        this(uf_id);
        nodes.add(new UNode(firstElement));
    }

    /**Returns the nodes of the UnionFind set
     * @return 
     */
    public ArrayList <UNode> getNodes(){ return nodes;}

    /** Reinitializes the Union Find object with a new Set consisted of an initial node
     @param uniqueElement the initial element of the new set
     */
    public void makeSet(int uniqueElement){
        nodes.add(new UNode(uniqueElement));
    }

    /** Finds the set to which x belongs to
     * @param target
     * @return  
     */
    public int find(UNode target){
        return find(target.getId());
    }

    /** Finds the set to which x belongs to
     * @param target 
     * @return 
     */
    public int find(int target){
        if(! nodes.contains(new UNode(target)) ) return Integer.MAX_VALUE;
        UNode targetPosition= getUNode(target);
        return find(targetPosition.getId(), targetPosition.getFather() );
    }

    /** Finds the set to which x belongs to by starting from the father node(recursively) */
    private int find(int x, int father){
       if (x!=father){
            UNode targetPosition= getUNode(father);
            if(targetPosition.hasFather())
                father=find(targetPosition.getId(), targetPosition.getFather() );
       }
     return father;
    }

    /**Unites two UNodes based on their id
     * @param x
     * @param y 
     * @return  
     */
    public boolean union(int x, int y){
        return union(new UNode(x), new UNode(y));
    }

    /** Unites two UNodes
     * @param x 
     * @param y
     * @return  
     */
    public boolean union(UNode x, UNode y){
       int r=find(x);
       int p=find(y);
       if (r!=p && r!=Integer.MAX_VALUE && p!=Integer.MAX_VALUE)
            return link(r, p);
        return false;
    }

    /** Links two UNodes  based on their id*/
    private boolean link(int x, int y){
        return link(getUNode(x), getUNode(y) );
    }

    /** Links two UNodes*/
    private boolean link(UNode x, UNode y){
        if (x.getRank()==y.getRank()){
            x.setFather(y.getId());
             y.setRank((Integer) (y.getRank()+1)  );
        }
        if (x.getRank()>y.getRank())
            y.setFather(x.getId());
        else x.setFather(y.getId());
        return true;
    }

    /** Adds a new UNode to the Union Find object with the predesigned father
     * @param node
     * @return  
     */
    public boolean addUNode(UNode node){
        if(nodes.contains(node))
            return false;
        nodes.add(node);
        /*if(node.hasFather()){
            while(node.getFather()!=UNode.noFather){
             //change all above ranks to +1
            }
        }*/
        return true;
    }

    /** Removes a UNode from the Union FInd Object
     * @param node
     * @return  
     */
    public boolean removeUNode(UNode node){
        if(!nodes.contains(node))
            return false;
        nodes.remove(node);
         /*if(node.hasFather()){
            while(node.getFather()!=UNode.noFather){
             //change all above ranks to -1 by changing them level-level (check fathers)
            }
        }*/
        return true;
    }

    /** Returns the UNode to whick the int id belongs to
     * @param id
     * @return  
     */
    public UNode getUNode(int id){
        return nodes.get(nodes.indexOf(new UNode(id)));
    }

    /** Prints the UNodes of the Object*/
    public void printNodes(){
        System.out.println(uf_id +": ");
        for(int i=0; i<nodes.size(); i++){
            UNode temp=nodes.get(i);
            if(temp.getFather()!=-Integer.MAX_VALUE)
                System.out.println("Id: " + temp.getId()+" Rank: "+ temp.getRank() + " Father: " + temp.getFather());
            else
                System.out.println("Id: " + temp.getId()+" Rank: "+ temp.getRank() + " Father: null");
        }
        System.out.println();
    }

}
