
package graphhandler;

import graphComponents.Graph;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;

/** GraphVisualizer class provides a simple/usable gui to represent a
 * Graph and it's Strongly Connected Components
 */
public class GraphVisualizer extends JFrame{

    /** The gui's components (frame and tabbedpanel)*/
    private JTabbedPane panel;
    private JFrame frame;

    /** The graphs opened by the gui*/
    HashMap<String, Graph> graphs;

    /** Returns an ImageIcon, or null if the path was invalid.
     * @param path
     * @param description 
     * @return  
     */
    public static ImageIcon createImageIcon(String path, String description) {
        java.net.URL imgURL = GraphVisualizer.class.getResource(path);
        if(imgURL != null){
            return new ImageIcon(imgURL , description);
        }
        return null;
    }

    /**Creates a window so that the user can choose a file for saving/loading and
     * then it returns that file
     * @param action
     * @param filterDescription
     * @return  
     */
    public static File chooseFile(String action, String filterDescription){
        JFileChooser fileChooser=new JFileChooser("Choose a file to...");
        //Add a custom file filter and disable the default
        fileChooser.addChoosableFileFilter(new FileChooserFilter(filterDescription));
        fileChooser.setAcceptAllFileFilterUsed(false);
        fileChooser.setDragEnabled(true);
        File temp=null;
        if (fileChooser.showDialog(new JFrame(), action) == JFileChooser.APPROVE_OPTION) {
                //System.out.println( (fileChooser.getSelectedFile()).getName());
                temp = fileChooser.getSelectedFile() ;
        }
        return temp;
    }

    /** Displays a panel that asks the user to choose a color and returns that color or null if the user presses cancel
     * @param title the title of the color dialog box
     * @return  
     */
    public static Color chooseColor(String title){
        JColorChooser jCChooser = new JColorChooser();
        return JColorChooser.showDialog(jCChooser, title, Color.yellow);
    }

    /**Default Constructor */
    public GraphVisualizer(){
        graphs=new HashMap<String, Graph>();
        //Frame setup
        frame= createFrame();
    }

    /**Creates/sets the frame of the window */
    private JFrame createFrame(){
        JFrame nFrame = new JFrame("Graph Visualizer");
        nFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        nFrame.setBackground(Color.WHITE);
        java.net.URL imgURL = getClass().getResource("images/graph.png");
        if (imgURL != null)
            nFrame.setIconImage(new ImageIcon(imgURL, "Graph").getImage());
        nFrame.setBounds(100, 100, 700, 730);
        nFrame.setBackground(Color.WHITE);
        nFrame.setLocationRelativeTo(null);
        nFrame.setLayout(new BorderLayout());
        nFrame.setVisible(true);
        nFrame = fillFrame(nFrame);
        return nFrame;
    }

    /**Fils the frame of the window with components (menubar, toolbar, tabbedpanel*/
    private JFrame fillFrame(JFrame frame){
        if (frame != null) {
            //MenuBar setup
            JMenuBar menu = createMenu();
            //Add menubar to the frame
            frame.setJMenuBar(menu);
            //Tabbed Panel setup
            panel = createTabbedPanel();
            //ToolBar
            JToolBar toolBar = createToolBar();
            //Add the Tabbed Panel and the Toolbar to the frame
            frame.getContentPane().add(toolBar, BorderLayout.NORTH);
            frame.getContentPane().add(panel, BorderLayout.CENTER);
            frame.setSize(frame.getWidth() + 1, frame.getHeight() + 1);
            frame.setSize(frame.getWidth() - 1, frame.getHeight() - 1);
            frame.repaint();
        }
        return frame;
    }

    /**Creates/sets the Tabbed panel of the window*/
    private JTabbedPane createTabbedPanel(){
        panel=new JTabbedPane();
        panel.setBackground(Color.GRAY);
        panel.addTab("Start", getDefaultTab());
        panel.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON2){
                         removeTab();
                    }
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        return panel;
    }

    /**Creates/sets the ToolBar */
    private JToolBar createToolBar(){
        JToolBar toolBar=new JToolBar();
        //Open Button
        JButton open= new JButton("Open", createImageIcon("images/open.png", "New Graph") );
        open.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        loadGraph();
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        toolBar.add(open);
        //Save Button
        JButton save= new JButton("Save", createImageIcon("images/save.png", "Save") );
        save.addMouseListener(new MouseListener() {
            @Override
            public void mouseClicked(MouseEvent e) {}
            @Override
            public void mousePressed(MouseEvent e) {
                if(e.getButton()==MouseEvent.BUTTON1){
                     save(null, "txt");
                }
            }
            @Override
            public void mouseReleased(MouseEvent e) {}
            @Override
            public void mouseEntered(MouseEvent e) {}
            @Override
            public void mouseExited(MouseEvent e) {}
        });
        toolBar.add(save);
        //Find SCC Menu Item
        JButton findScc= new JButton("Find SCC",createImageIcon("images/execute.png", "Find-SCC") );
        findScc.addMouseListener(new MouseListener() {
            @Override
            public void mouseClicked(MouseEvent e) {}
            @Override
            public void mousePressed(MouseEvent e) {
                if(e.getButton()==MouseEvent.BUTTON1){
                    showGraph(Graph.SHOW_SCC);
                }
            }
            @Override
            public void mouseReleased(MouseEvent e) {}
            @Override
            public void mouseEntered(MouseEvent e) {}
            @Override
            public void mouseExited(MouseEvent e) {}
        });
        toolBar.add(findScc);
        //Run Kruskal Menu Item
        JButton runKruskal= new JButton("Run Kruskal",createImageIcon("images/executek.png", "Find-SCC") );
        runKruskal.addMouseListener(new MouseListener() {
            @Override
            public void mouseClicked(MouseEvent e) {}
            @Override
            public void mousePressed(MouseEvent e) {
                if(e.getButton()==MouseEvent.BUTTON1){
                    showGraph(Graph.SHOW_KRUSKAL);
                }
            }
            @Override
            public void mouseReleased(MouseEvent e) {}
            @Override
            public void mouseEntered(MouseEvent e) {}
            @Override
            public void mouseExited(MouseEvent e) {}
        });
        toolBar.add(runKruskal);
        //Add the "Show SCC" JComboBox
        ArrayList <String> sccNodes=new ArrayList <String>();
        sccNodes.add("Initial Graph");
        sccNodes.add("Graph's SCC");
        sccNodes.add("Shortest Path");
        Graph target = getSelectedGraph();
        if (target!=null){
                ArrayList<Graph> scc = target.findSCC();
                for (int i = 0; i < scc.size(); i++)
                    sccNodes.add(scc.get(i).nodesToString());
        }
        JComboBox selectSCC= new JComboBox(sccNodes.toArray());
        selectSCC.setBackground(Color.WHITE);
        selectSCC.addActionListener(new ActionListener(){
            @Override
                public void actionPerformed(ActionEvent e) {
                     showGraph(((JComboBox) e.getSource()).getSelectedIndex());
                }

        });
        toolBar.add(selectSCC);

        toolBar.setSize(350, 10);
        toolBar.setBorderPainted(true);
        toolBar.setFloatable(false);
        toolBar.setRollover(true);
        return toolBar;
    }

    /**Saves the graph in a tab to the target file*/
    private boolean save(File file, String ext) {
        Graph target = getSelectedGraph();
        if (target!=null && FileChooserFilter.isAcceptableExtension(ext))
            target.saveAs(file, ext);
        else if(!FileChooserFilter.isAcceptableExtension(ext) )
             JOptionPane.showMessageDialog(frame, "Error in file " + file.getName() + " type.\n ", "Error", JOptionPane.ERROR_MESSAGE);
        return true;
    }

    /**Creates/sets the Edit menu and its items*/
    private JMenu createEditMenu(){
        JMenu edit=new JMenu("Edit", true);
        //Find SCC Menu Item
        JMenuItem scc= new JMenuItem("Find SCC", createImageIcon("images/execute.png", "Find-SCC") );
        scc.addActionListener(new ActionListener(){
            @Override
                public void actionPerformed(ActionEvent e) {
                        if(e.getModifiers()!=4 && e.getModifiers()!=8 && e.getModifiers()!=0){ //if it's not because of a right/middle click or a keyboard action key
                            showGraph(Graph.SHOW_SCC);
                        }
                }
        });
        scc.setMnemonic('F');
        scc.setAccelerator(KeyStroke.getKeyStroke("ctrl F"));
        edit.add(scc);
        //Run Kruskal Menu Item
        JMenuItem kruskal= new JMenuItem("Run Kruskal", createImageIcon("images/executek.png", "Find-SCC") );
        kruskal.addActionListener(new ActionListener(){
            @Override
                public void actionPerformed(ActionEvent e) {
                        if(e.getModifiers()!=4 && e.getModifiers()!=8 && e.getModifiers()!=0){ //if it's not because of a right/middle click or a keyboard action key
                            showGraph(Graph.SHOW_KRUSKAL);
                        }
                }
        });
        kruskal.setMnemonic('K');
        kruskal.setAccelerator(KeyStroke.getKeyStroke("ctrl K"));
        edit.add(kruskal);
        //Vertex Color picker Menu Item
        JMenuItem cellColor= new JMenuItem("Vertex Color", createImageIcon("images/cellcolor.png", "Vertex Color") );
        cellColor.addActionListener(new ActionListener(){
            @Override
                public void actionPerformed(ActionEvent e) {
                        if(e.getModifiers()!=4 && e.getModifiers()!=8 && e.getModifiers()!=0){ //if it's not because of a right/middle click or a keyboard action key
                            updateGraph(-1, -1, chooseColor("Choose a Color for the Nodes"), null);
                        }
                }
        });
        cellColor.setMnemonic('V');
        cellColor.setAccelerator(KeyStroke.getKeyStroke("ctrl V"));
        edit.add(cellColor);
        
        //Edge Color picker Menu Item
        JMenuItem edgeColor= new JMenuItem("Edge Color", createImageIcon("images/edgecolor.png", "Edge Color") );
        edgeColor.addActionListener(new ActionListener(){
            @Override
                public void actionPerformed(ActionEvent e) {
                        if(e.getModifiers()!=4 && e.getModifiers()!=8 && e.getModifiers()!=0){ //if it's not because of a right/middle click or a keyboard action key
                            updateGraph(-1, -1, null, chooseColor("Choose a Color for the Edges"));
                        }
                }
        });
        edgeColor.setMnemonic('E');
        edgeColor.setAccelerator(KeyStroke.getKeyStroke("ctrl E"));
        edit.add(edgeColor);
        //Choose Layout Menu Item
        JMenu chooseLayout= new JMenu("Graph Layout:");
        //Circle Layout
        JMenuItem circle =new JMenuItem("Circle");
        circle.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        updateGraph(VGraph.VGRAPH_CIRCLE_LAYOUT, -1, null, null);
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        chooseLayout.add(circle);
        //Tilt Layout
        JMenuItem tilt =new JMenuItem("Tilt");
        tilt.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        updateGraph(VGraph.VGRAPH_TILT_LAYOUT, -1, null, null);
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        chooseLayout.add(tilt);
        //Random Layout
        JMenuItem random =new JMenuItem("Random");
        random.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        updateGraph(VGraph.VGRAPH_RANDOM_LAYOUT, -1, null, null);
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        chooseLayout.add(random);
        //Fast Organic Layout
        JMenuItem fastOrg =new JMenuItem("Fast Organic");
        fastOrg.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        updateGraph(VGraph.VGRAPH_FAST_ORGANIC_LAYOUT, -1, null, null);
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        chooseLayout.add(fastOrg);
        //Organic Layout
        JMenuItem organic =new JMenuItem("Organic");
        organic.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        updateGraph(VGraph.VGRAPH_ORGANIC_LAYOUT, -1, null, null);
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        chooseLayout.add(organic);
        edit.add(chooseLayout);
        //Choose Layout Menu Item
        JMenu EdgeType= new JMenu("Edge Type:");
        //Orthogonal Edges
        JMenuItem orthogonal =new JMenuItem("Orthogonal");
        orthogonal.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        updateGraph(-1, VGraph.VGRAPH_ORTHOGONAL_EDGES, null, null);
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        EdgeType.add(orthogonal);
        //Bezier Edges
        JMenuItem bezier =new JMenuItem("Bezier");
        bezier.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        updateGraph(-1, VGraph.VGRAPH_BEZIER_EDGES, null, null);
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        EdgeType.add(bezier);
        //Spline Edges
        JMenuItem spline =new JMenuItem("Spline");
        spline.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        updateGraph(-1, VGraph.VGRAPH_SPLINE_EDGES, null, null);
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        EdgeType.add(spline);
        edit.add(EdgeType);
        return edit;
    }

    /**Creates/sets the About menu and its items*/
    private JMenu createAboutMenu(){
        JMenu about= new JMenu("About", true);
        //Help Button
        JMenuItem help= new JMenuItem("Help", createImageIcon("images/info.png", "Help") );
        help.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if (e.getButton() == MouseEvent.BUTTON1) {
                        createHelpFrame();
                    }
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        about.add(help);
        //Info Button
        JMenuItem info= new JMenuItem("Info", createImageIcon("images/home.png", "Info") );
        info.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if (e.getButton() == MouseEvent.BUTTON1) {
                        createInfoFrame();
                    }
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        about.add(info);

        return about;
    }

    /**Creates/Sets the info Frame*/
    private JFrame createHelpFrame(){
        JFrame window = new JFrame("Help");
        window.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        java.net.URL imgURL = getClass().getResource("images/graph.png");
        if (imgURL != null) {
            window.setIconImage(new ImageIcon(imgURL, "Graph").getImage());
        }
        window.setLayout(new BorderLayout(7,7));
        window.getContentPane().add(new JLabel("Quick Tips:"), BorderLayout.NORTH);
        JTextArea tips= new JTextArea("1) Close a tab by simply clicking it with the mouse's middle click.\n" +
                                      "2) Save your graph as a .txt, .png, .jpg, .jpeg file through the Save As button in the File menu.\n" +
                                      "3) Open a new Graph File either through the open button in the file menu or by clicking the image in the Start tab" +
                                      "or simply by pressing ctrl+o .\n" +
                                      "4) Begin the Finding of the SCC in the graph by clicking the equivalent button in the Edit menu or by pressing ctrl + F.\n" +
                                      "5) You can choose (via the Edit Menu) the layout and the edges style you want the graphs to be represented in.\n" +
                                      "6) GraphVisualizer only accepts .txt files as graph input so it 'hides' all the other extensions when choosing a new file for safety.\n"+
                                      "7) After finding the SCC of the graph you can navigate between the Initial Graph, the SCC and the SCC components via the ComboBox in the toolbar.\n" +
                                      "8) You can pick a custom color for the graph's cells or for the edges via the menu items in the Edit menu.\n" +
                                      "9) When the SCC are found for a graph they will appear with different/random colors.\n" +
                                      "10) When a new Graph is loaded or when the Layout changes a popup progressBar will appear that shows you the progress in loading the Visual part of the Graph.\n" +
                                      "11) Don't forget to check the Info in the About menu. :).");
        tips.setWrapStyleWord(true);
        window.getContentPane().add(new JScrollPane(tips, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED),BorderLayout.CENTER);
        window.setVisible(true);
        window.setBounds(150, 150, 150, 200);
        window.setResizable(false);
        window.pack();
        return window;
    }

    /**Creates/Sets the info Frame*/
    private JFrame createInfoFrame(){
        JFrame window = new JFrame("Info");
        window.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        java.net.URL imgURL = getClass().getResource("images/graph.png");
        if (imgURL != null) {
            window.setIconImage(new ImageIcon(imgURL, "Graph").getImage());
        }
        window.setLayout(new BorderLayout(5,5));
        window.getContentPane().add(new JLabel(createImageIcon("images/weeme.jpg", "Paulos")), BorderLayout.WEST);
        window.getContentPane().add(new JLabel(createImageIcon("images/weeme2.jpg", "Dimitris")), BorderLayout.EAST);
        JPanel names=new JPanel(new BorderLayout(5,5));
        names.add(new JLabel("Paulos Sarbinowski                 "), BorderLayout.WEST);
        names.add(new JLabel("Dimitris Moudilos"), BorderLayout.EAST);
        window.getContentPane().add(names, BorderLayout.SOUTH);
        window.setVisible(true);
        window.setBounds(150, 150, 200, 300);
        window.setResizable(false);
        window.pack();
        return window;
    }

    /**Creates/sets the File menu and its items*/
    private JMenu createFileMenu(){
        JMenu file= new JMenu("File", true);
        //Open Menu Item
        JMenuItem open= new JMenuItem("Open", createImageIcon("images/open.png", "Open") );
        open.addActionListener(new ActionListener(){
            @Override
                public void actionPerformed(ActionEvent e) {
                    if(e.getModifiers()!=4 && e.getModifiers()!=8 && e.getModifiers()!=0){//if it's not because of a right/middle click or a keyboard action key
                        loadGraph();
                    }
                }
        });
        open.setMnemonic('O');
        open.setAccelerator(KeyStroke.getKeyStroke("ctrl O"));
        open.setSize(35, 15);
        file.add(open);
        //Save Menu Item
        JMenuItem save= new JMenuItem("Save", createImageIcon("images/save.png", "Save") );
        save.addActionListener(new ActionListener(){
            @Override
                public void actionPerformed(ActionEvent e) {
                    if(e.getModifiers()!=4 && e.getModifiers()!=8 && e.getModifiers()!=0){//if it's not because of a right/middle click or a keyboard action key
                         save(null, "txt");
                    }
                }
        });
        save.setMnemonic('S');
        save.setAccelerator(KeyStroke.getKeyStroke("ctrl S"));
        file.add(save);
        //Save As Menu Item
        JMenuItem saveas= new JMenuItem("Save as..", createImageIcon("images/saveas.png", "Save-As") );
        saveas.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1){
                         File file =chooseFile("Save", ".txt, .png, .jpeg, .jpg");
                         if(file!=null)
                              save(file, FileChooserFilter.getExtension(file));
                    }
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        file.add(saveas);
        //Quit Menu Item
        JMenuItem quit= new JMenuItem("Quit", createImageIcon("images/quit.png", "Quit") );
        quit.addActionListener(new ActionListener(){
            @Override
                public void actionPerformed(ActionEvent e) {
                    if(e.getModifiers()!=4 && e.getModifiers()!=8 && e.getModifiers()!=0)//if it's not because of a right/middle click or a keyboard action key
                        System.exit(0);
                }
        });
        quit.setMnemonic('Q');
        quit.setAccelerator(KeyStroke.getKeyStroke("ctrl Q"));
        file.add(quit);
        return file;
    }

    /**Removes a tab from the tab panel*/
    private void removeTab(){
        if(panel != null && panel.getTabCount()!=0){
            String selection = panel.getSelectedComponent().getName();
            if (selection!=null && graphs.containsKey(selection))
                graphs.remove(selection);
             panel.remove(panel.getSelectedIndex());
        }
    }

    /**Returns a default tab with the basic buttons
     * @return 
     */
    public JPanel getDefaultTab(){
        JPanel nPanel = new JPanel(new GridBagLayout());
        nPanel.setBackground(Color.WHITE);
        GridBagConstraints cons=new GridBagConstraints();
        //Load Icon
        JLabel icon= new JLabel(createImageIcon("images/initial.png", "Load"));
        icon.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        loadGraph();
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        cons.gridx = 0; cons.gridy = 0;
        nPanel.add(icon, cons);
        //Load Button
        JButton open= new JButton("Load Graph");
        open.addMouseListener(new MouseListener() {
            @Override
                public void mouseClicked(MouseEvent e) {}
            @Override
                public void mousePressed(MouseEvent e) {
                    if(e.getButton()==MouseEvent.BUTTON1)
                        loadGraph();
                }
            @Override
                public void mouseReleased(MouseEvent e) {}
            @Override
                public void mouseEntered(MouseEvent e) {}
            @Override
                public void mouseExited(MouseEvent e) {}
        });
        cons.fill = GridBagConstraints.VERTICAL;
        cons.gridx = 0; cons.gridy = 1;
        nPanel.add(open, cons);
        return nPanel;
    }

    /**Creates a new Graph after it gets the origin file from the user*/
    public void loadGraph() {
        File file = chooseFile("Open", "");
        try {
            if (file != null )
                loadGraph(file);
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(frame, "Error in reading file" + file.getName() + ".\n ", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    /**Creates a new Graph tab based on the file
     * @param file
     * @throws IOException  
     */
    public void loadGraph(File file) throws IOException{
        Graph graph= new Graph(file);
        if(graphs.containsValue(graph)){
            JOptionPane.showMessageDialog(frame, "Error, file " + file.getName() + " is already opened.", "Error", JOptionPane.ERROR_MESSAGE);
        }else if(!graph.readFailed() ){
            frame.getContentPane().remove(panel);
            panel.addTab(file.getName(), graph.getGraphPanel(Graph.SHOW_INITIAL_GRAPH));
            graphs.put(file.getName(), graph);
            panel.setSelectedIndex(panel.getComponentCount()-1);
            frame.getContentPane().add(panel, BorderLayout.CENTER);
            frame.repaint();
        }
    }

    /**Creates/sets the MenuBar*/
    private JMenuBar createMenu(){
        JMenuBar menu=new JMenuBar();
        JMenu file= createFileMenu();
        menu.add(file);
        JMenu edit = createEditMenu();
        menu.add(edit);
        JMenu about = createAboutMenu();
        menu.add(about);

        return menu;
    }

    /** Activates when a different Graph is selected in the Combo Box. */
    private void showGraph(int selection){
        Graph target = getSelectedGraph();
        if (target!=null)
            if(selection==Graph.SHOW_SCC)
                reloadGraphTab(target, target.getGraphPanel(selection), true); //True to reloadAll of the components so that the toolbar is synchronized
            else if(selection==Graph.SHOW_KRUSKAL)
                reloadGraphTab(target, target.Kruskal().getGraphPanel(selection), true);
            else
                reloadGraphTab(target, target.getGraphPanel(selection), false);
    }

    /** Updates Graph after activating the appropriate menuItem from the Edit Menu.*/
    private void updateGraph(int vLayout, int edgeStyle, Color vColor, Color eColor){
        VGraph.setCharacteristics(vLayout, edgeStyle, vColor, eColor);
        Graph target = getSelectedGraph();
        if (target!=null)
             target.updateVisualGraph();
    }

    /** Returns the Graph selected each time by the user or null if none has been selected.*/
    private Graph getSelectedGraph(){
        if (panel != null && panel.getTabCount()!=0) {
            String selection = panel.getSelectedComponent().getName();
            if (selection!=null && graphs.containsKey(selection))
                return graphs.get(selection);
        }
        return null;
    }

    /** Handles the reloading of the gui after a change is executed so that the gui corresponds to the new data.
     * @param target The target graph who's tab will be reloaded.
     * @param component The new component that will be loaded in the target's tab.
     * @param reloadAll Boolean that suggests if the function will reload both toolbar and tabbedpanel(if true) or only the tabbedPanel(if false)
     */
    private void reloadGraphTab(Graph target, Component component, boolean reloadAll){
            if (reloadAll)
                frame.getContentPane().removeAll();     //remove all previous contentPane elements
            else
                frame.getContentPane().remove(panel);     //remove all previous contentPane elements
            int sindex = panel.getSelectedIndex();
            panel.removeTabAt(sindex);           //remove selected tab to refresh
            panel.insertTab(target.getOriginFile().getName(), null, component, null, sindex);  //add the refreshed Tab
            panel.setSelectedIndex(sindex);
            //Add again the components to the Frame's ContentPane
            if(reloadAll)
                frame.getContentPane().add(createToolBar(), BorderLayout.NORTH);
            frame.getContentPane().add(panel, BorderLayout.CENTER);
            frame.setSize(frame.getWidth() + 1, frame.getHeight() + 1);
            frame.setSize(frame.getWidth() - 1, frame.getHeight() - 1);
            frame.repaint();
    }

}
