

package graphhandler;

import graphComponents.GraphSCC;
import graphComponents.Graph;
import graphComponents.Edge;
import com.jgraph.layout.*;
import com.jgraph.layout.graph.JGraphSimpleLayout;
import com.jgraph.layout.organic.*;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.beans.*;
import java.io.*;
import java.util.*;
import javax.imageio.ImageIO;
import javax.swing.*;
import org.jgraph.JGraph;
import org.jgraph.graph.*;

/**
 * VGraph class.
 * Represents visually (utilizing the JGraph library) a graph
 */
public class VGraph {

    /** The JGraph object that handles the representation. */
    private JGraph visualGraph;
    
    /** Necessary for the proper function of JGraph*/
    private JGraphFacade vGraphFacade;
    private JGraphLayout vGraphLayout;

    private boolean isSCC;
    private JFrame frame; /** The frame that contains the visualGraph */
    private final JProgressBar progressBar = new JProgressBar(0, 100);

    /** Constants used for describing the Layout and the Edge Style in the JGraph*/
    public static final int VGRAPH_CIRCLE_LAYOUT=0;
    public static final int VGRAPH_TILT_LAYOUT=1;
    public static final int VGRAPH_RANDOM_LAYOUT=2;
    public static final int VGRAPH_FAST_ORGANIC_LAYOUT=3;
    public static final int VGRAPH_ORGANIC_LAYOUT=4;
    public static final int VGRAPH_ORTHOGONAL_EDGES=0;
    public static final int VGRAPH_BEZIER_EDGES=1;
    public static final int VGRAPH_SPLINE_EDGES=2;

    /** Static members that should be the same for all of the VGraph's*/
    private static int vLayout, edgeStyle, vWidth, vHeight;
    private static Color vColor, eColor;
    private static boolean changeLayout, changeEdgeStyle, changeColor;

    /** Empty Constructor. Builds a VGraph representing a new empty Graph */
    public VGraph(){
        this(new Graph());
    }

    /** Constructor that builds a VGraph based on the initial 'Graph' graph
     * @param graph 
     */
    public VGraph(Graph graph){
        this(graph, null);
    }
    
    /** Constructor based on a vector filled with graph objects that represents them as specific nodes
     * @param originGraph is the graph on which the vGraph will be based
     * @param graphs is a vector that contains the scc of the graph
     */
    public VGraph(Graph originGraph, GraphSCC graphs){
        if(vColor == null) vColor=new Color(255, 150, 0); //Dark Orange
        if(eColor == null) eColor=Color.DARK_GRAY;
        if( vHeight<1 )vHeight=800;
        if( vWidth<1 ) vWidth=800;
        changeEdgeStyle=changeLayout=changeColor=true;
        
        GraphModel model = new DefaultGraphModel();
        GraphLayoutCache view = new GraphLayoutCache(model, new DefaultCellViewFactory());

        visualGraph = new JGraph(model, view);
        visualGraph.setCloneable(true);
        visualGraph.setDisconnectable(false);
        visualGraph.setConnectable(false);
        visualGraph.setBendable(true);
        visualGraph.setAutoResizeGraph(true);
        visualGraph.setInvokesStopCellEditing(true);
        visualGraph.setJumpToDefaultPort(true);
        visualGraph.setVisible(true);
        visualGraph.setAntiAliased(true);
        visualGraph.getGraphLayoutCache().setAutoSizeOnValueChange(true);
        DefaultGraphCell[] cells;
        if(graphs==null)
            cells = fillCells(originGraph);
        else
            cells = fillCells(originGraph, graphs);
        visualGraph.getGraphLayoutCache().insert(cells);
        vGraphFacade = new JGraphFacade(visualGraph);
        updateVGraph(vWidth, vHeight);
    }

    /** Fills a DefaultGraphCell array with vertexs/Edges based on the SCC's of the graph
     * @param originGraph is the graph on which the vGraph will be based
     * @param graphs is a vector that contains the scc of the graph
     * @return  
     */
    public DefaultGraphCell[] fillCells(Graph originGraph, GraphSCC graphs){
        isSCC=true;
        DefaultGraphCell[] cells= new DefaultGraphCell[graphs.size()+ graphs.getSCCEdgeCount(originGraph)];
        HashMap<Integer, Integer> vertToCell= new HashMap<Integer, Integer>();
        //Add the vertexs/Strongly Connected Components to the DefaultGraphCell array
        int cellNum=0;
        Random rGen=new Random();
        for (int i=0; i<graphs.size(); i++){ //while it has next vertexs
            cells[cellNum]=getGraphCell(graphs.get(i).nodesToString(), new Rectangle2D.Double( (double) rGen.nextInt(vWidth),(double) rGen.nextInt(vHeight), 40, 20), vColor, true);
            vertToCell.put(i, cellNum++);
        }
        
        //Add the edges between the SCC to the DefaultGraphCell array by checking each time if the corresponding edge already exists (in sccEdges)
        HashMap <Integer, LinkedList <Integer>> sccEdges= graphs.getSCCEdges(originGraph);
        Set origins = sccEdges.keySet();
        LinkedList targets; int id1, id2;
        for(Iterator iter = origins.iterator(); iter.hasNext();){
                id1 =(Integer) iter.next();
                targets = sccEdges.get(id1);
                for(int i=0;targets!=null &&  i< targets.size(); i++){
                    id2 =(Integer) targets.get(i);
                    cells[cellNum++] = getGraphEdge(cells[vertToCell.get(id1)], cells[vertToCell.get(id2)], eColor);
                }
        }                

        return cells;
    }

    /** Fills a DefaultGraphCell array with vertexs/Edges based on the Graph
     * @param graph is the graph on which the vGraph will be based
     * @return  
     */
    public DefaultGraphCell[] fillCells(Graph graph){
        isSCC=false;
        DefaultGraphCell[] cells= new DefaultGraphCell[(graph.getNodesNum()+graph.getEdgeCount())];
        HashMap<Integer, Integer> vertToCell= new HashMap<Integer, Integer>();
        //Add the vertexs of the graph to the DefaultGraphCell array
        Random rGen=new Random();
        Set verts = graph.getVertexs().keySet();int cellNum=0;
        for (Iterator iter = verts.iterator(); iter.hasNext() && cellNum < cells.length;){ //while it has next vertexs
            int vert= (Integer) iter.next();
            cells[cellNum]=getGraphCell(("" + vert),new Rectangle2D.Double((double) rGen.nextInt(vWidth), (double) rGen.nextInt(vHeight), 40, 20), vColor, true);
            vertToCell.put(vert, cellNum++);
        }
        //Add the edges to the DefaultGraphCell array
        LinkedList list; int id1, id2;
        for(Iterator iter= verts.iterator() ; iter.hasNext(); ){
            id1 =(Integer) iter.next();
            list=(LinkedList) graph.getAdjacencyMap().get(id1);
            for(int i=0;list!=null &&  i< list.size(); i++){
                id2 =((Edge) list.get(i)).getTo();
                cells[cellNum++] = getGraphEdge(cells[vertToCell.get(id1)], cells[vertToCell.get(id2)], eColor);
            }
         }
        return cells;
    }

    /** Returns a DefaultEdge based on the arguments
     * @param source the cell from which the edge will begin
     * @param target the cell to which the edge will arive
     * @param eColor the color of the edge
     */
    private DefaultEdge getGraphEdge(DefaultGraphCell source, DefaultGraphCell target, Color eColor){
        DefaultEdge edge = new DefaultEdge();
        edge.setSource(source.getChildAt(0));
        edge.setTarget(target.getChildAt(0));
        int arrow = GraphConstants.ARROW_SIMPLE;
        GraphConstants.setLineEnd(edge.getAttributes(), arrow);
        GraphConstants.setEditable(edge.getAttributes(), false);
        GraphConstants.setEndFill(edge.getAttributes(), false);
        GraphConstants.setLineColor(edge.getAttributes(), eColor);
        GraphConstants.setLineStyle(edge.getAttributes(), GraphConstants.STYLE_ORTHOGONAL);
        GraphConstants.setRouting(edge.getAttributes(), GraphConstants.ROUTING_SIMPLE);
        return edge;
    }

    /** Returns a GraphCell that represents a vertex as described in the arguments
     * @param name the name that will be displayed in the cell
     * @param bounds the coordinates of the cell
     * @param color the color of the cell
     * @param opaque if it's true the cell will be opaque
     */
    private DefaultGraphCell getGraphCell(String name, Rectangle2D.Double bounds, Color color, boolean opaque){
        DefaultGraphCell cell= new DefaultGraphCell(name);
        GraphConstants.setAutoSize(cell.getAttributes(), true);
        GraphConstants.setEditable(cell.getAttributes(), false);
        GraphConstants.setBounds(cell.getAttributes(), bounds);
        GraphConstants.setGradientColor( cell.getAttributes(), color);
        GraphConstants.setOpaque(cell.getAttributes(), opaque);
        //GraphConstants.setBorder(cell.getAttributes(), BorderFactory.createLineBorder(visualGraph.getBackground(), 2));
        GraphConstants.setBorder(cell.getAttributes(), BorderFactory.createRaisedBevelBorder());
        GraphConstants.setInset(cell.getAttributes(), 1);
        DefaultPort port = new DefaultPort();
        cell.add(port);
        return cell;
    }

    /** Returns the JGraph object
     * @return 
     */
    public JGraph getVGraph(){ return visualGraph;}

    /** Sets the JGraph object
     * @param visualGraph 
     */
    public void setVGraph(JGraph visualGraph){
        if(visualGraph!=null)
            this.visualGraph=visualGraph;
    }

    /** Updates the Graph with new Layout/Edge design \
     * @param width is the width of the JGraph
     * @param height is the height of the JGraph
     */
    public void updateVGraph(int width, int height){
        vWidth= width;
        vHeight= height;
        //updateJGraph();
        SwingWorker worker = updateWorker();
        worker.addPropertyChangeListener(  new PropertyChangeListener() {
            @Override
             public  void propertyChange(PropertyChangeEvent evt) {
                 if ("progress".equals(evt.getPropertyName())) {
                     progressBar.setValue((Integer)evt.getNewValue());
                 }
             }
        });
        worker.execute();
        
    }

    /** Used to break into a second thread the task of updating the VGraph so that the gui doesn't get stuck (multitasking)*/
    private SwingWorker updateWorker(){
        return new SwingWorker<Void, Void>() {
            @Override
                public void done() {
                       setProgress(91);
                       visualGraph.getGraphLayoutCache().edit(vGraphFacade.createNestedMap(true, true));
                       visualGraph.getGraphLayoutCache().refresh(visualGraph.getGraphLayoutCache().getAllViews(), true);
                       visualGraph.addOffscreenDirty(GraphLayoutCache.getBounds(visualGraph.getGraphLayoutCache().getAllViews()));
                       visualGraph.repaint();
                       setProgress(99);
                       changeLayout=changeEdgeStyle=changeColor=false; //reset all flags
                       stopLoadingFrame();
                }

            @Override
                protected Void doInBackground() throws Exception {
                       try {
                            JGraph graph= getVGraph();
                            startLoadingFrame();
                            setProgress(10);
                            JGraphLayout vGraphLayout=null;
                            if(isValidLayout(getVLayout()) && changeLayout){
                                    setProgress(22);
                                    if(getVLayout()==VGRAPH_CIRCLE_LAYOUT){
                                        vGraphLayout = new JGraphSimpleLayout(JGraphSimpleLayout.TYPE_CIRCLE, vWidth, vHeight);
                                    } else if(getVLayout()==VGRAPH_TILT_LAYOUT){
                                        vGraphLayout = new JGraphSimpleLayout(JGraphSimpleLayout.TYPE_TILT, vWidth, vHeight);
                                    } else if(getVLayout()==VGRAPH_RANDOM_LAYOUT){
                                        vGraphLayout = new JGraphSimpleLayout(JGraphSimpleLayout.TYPE_RANDOM, vWidth, vHeight);
                                    }else if(getVLayout()==VGRAPH_FAST_ORGANIC_LAYOUT){
                                        vGraphLayout = new JGraphFastOrganicLayout();
                                    }else if(getVLayout()==VGRAPH_ORGANIC_LAYOUT){
                                        vGraphLayout = new JGraphOrganicLayout(graph.getVisibleRect());
                                        ((JGraphOrganicLayout) vGraphLayout).setFineTuning(true);
                                        ((JGraphOrganicLayout) vGraphLayout).setOptimizeBorderLine(true);
                                        ((JGraphOrganicLayout) vGraphLayout).setOptimizeEdgeCrossing(true);
                                        ((JGraphOrganicLayout) vGraphLayout).setOptimizeEdgeDistance(true);
                                        ((JGraphOrganicLayout) vGraphLayout).setOptimizeEdgeLength(true);
                                        ((JGraphOrganicLayout) vGraphLayout).setOptimizeNodeDistribution(true);
                                    }setProgress(37);
                                     vGraphLayout.run(getvGraphFacade());
                                     setVGraphLayout(vGraphLayout);
                                     setProgress(62);
                            }setProgress(68);
                            if(isValidEdgeStyle(getEdgeStyle()) && changeEdgeStyle){
                                    for (CellView cView : graph.getGraphLayoutCache().getAllViews()) {
                                        if (getEdgeStyle()==VGRAPH_SPLINE_EDGES) {
                                            GraphConstants.setLineStyle(cView.getAttributes(), GraphConstants.STYLE_SPLINE);
                                        } else if (getEdgeStyle()==VGRAPH_BEZIER_EDGES) {
                                            GraphConstants.setLineStyle(cView.getAttributes(), GraphConstants.STYLE_BEZIER);
                                        } else if (getEdgeStyle()==VGRAPH_ORTHOGONAL_EDGES) {
                                            GraphConstants.setLineStyle(cView.getAttributes(), GraphConstants.STYLE_ORTHOGONAL);
                                        }
                                        GraphConstants.setRouting(cView.getAttributes(), GraphConstants.ROUTING_SIMPLE);
                                        graph.getGraphLayoutCache().edit(graph.getModel().getAttributes(cView));
                                    }
                            }setProgress(83);
                            if(changeColor){
                                    Random rand= new Random();
                                    for (CellView cView : graph.getGraphLayoutCache().getAllViews()) {
                                            if(cView instanceof EdgeView)
                                                 GraphConstants.setLineColor(cView.getAttributes(), getEdgeColor());
                                            else{
                                                if(isSCC)
                                                    GraphConstants.setGradientColor(cView.getAttributes(), new Color(rand.nextInt(220), rand.nextInt(220), rand.nextInt(220)));
                                                else
                                                    GraphConstants.setGradientColor(cView.getAttributes(), getVertexColor());
                                            }
                                            graph.getGraphLayoutCache().edit(graph.getModel().getAttributes(cView));
                                    }setProgress(88);
                            }
                            setVGraph(graph);
                       }catch(Exception e){
                            JOptionPane.showMessageDialog(new JFrame(), "Error in Graph's Visual Computation.\n" ,"Error", JOptionPane.ERROR_MESSAGE);
                       }
                       return null;
                }
            };
    }

    /**Exports the Graph as a jpg or png to a file
     * @param target is the File the image will be exported to
     * @param imgType is the file's extension
     */
    public void exportImage(File target, String imgType) {
        if( visualGraph!=null){
            try {
                boolean overWrite = true;
                if(target.exists())
                    overWrite = (JOptionPane.showConfirmDialog(new JFrame(), "File already exists. Overwrite anyway?",
                            "Overwrite", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)==JOptionPane.YES_OPTION) ;

                if(overWrite){
                    BufferedOutputStream out = new  BufferedOutputStream(new FileOutputStream(target)); // Replace with your output stream
                    Color bg;
                    if(imgType.equalsIgnoreCase("png"))
                        bg = null;                          //if it is supported by the file format make the background transparent
                    else
                        bg = visualGraph.getBackground();   //else use the graph's background
                    BufferedImage img = visualGraph.getImage(bg, 3);
                    ImageIO.write(img, imgType, target);
                    out.flush();
                    out.close();
                }
            } catch (IOException ex) {
                JOptionPane.showMessageDialog(new JFrame(), "Error in exporting Graph.", "Error", JOptionPane.ERROR_MESSAGE);
            }
            
        }
    }

    /** Returns true if the vLayout parameter belongs to one of the approved Layout constants
     * @param vLayout
     * @return  
     */
    public static boolean isValidLayout(int vLayout){
        return(vLayout>=VGRAPH_CIRCLE_LAYOUT && vLayout<=VGRAPH_ORGANIC_LAYOUT);
    }

    /** Returns true if the edgeStyle parameter belongs to one of the approved Edge style constants
     * @param edgeStyle
     * @return  
     */
    public static boolean isValidEdgeStyle(int edgeStyle){
        return(edgeStyle>=VGRAPH_ORTHOGONAL_EDGES && edgeStyle<=VGRAPH_SPLINE_EDGES);
    }

    /** Sets the Layout for all the VGraphs
     * @param vLayout 
     */
    public static void setVLayout(int vLayout){
        if(isValidLayout(vLayout) && VGraph.vLayout!=vLayout){
            VGraph.vLayout=vLayout;
            changeLayout=true;
        }
    }

    /** Sets the Edge Style for all the VGraphs
     * @param edgeStyle 
     */
    public static void setEdgeStyle(int edgeStyle){
        if(isValidEdgeStyle(edgeStyle) && VGraph.edgeStyle!=edgeStyle){
            VGraph.edgeStyle=edgeStyle;
            changeEdgeStyle=true;
        }
    }

    /** Returns the Layout for all the VGraphs
     * @return 
     */
    public static int getVLayout(){
            return vLayout;
    }

    /** Returns the Edge Style for all the VGraphs
     * @return 
     */
    public static int getEdgeStyle(){
            return edgeStyle;
    }

    /** Sets the Color for all the vertexs in VGraphs
     * @param vColor 
     */
    public static void setVertexColor(Color vColor){
        if(vColor!=null && VGraph.vColor!=vColor){
            VGraph.vColor=vColor;
            changeColor=true;
        }
    }

    /** Sets the Color for all the edges in VGraphs
     * @param eColor 
     */
    public static void setEdgeColor(Color eColor){
        if(eColor!=null && VGraph.eColor!=eColor){
            VGraph.eColor=eColor;
            changeColor=true;
        }
    }

    /** Returns the JGraph's Facade
     * @return 
     */
    public JGraphFacade getvGraphFacade(){
        return vGraphFacade;
    }

    /** Returns the JGraph's Layout
     * @return 
     */
    public JGraphLayout getVGraphLayout(){
        return vGraphLayout;
    }

    /** Sets the JGraph's Layout
     * @param layout 
     */
    public void setVGraphLayout(JGraphLayout layout){
        if(layout!=null)
            vGraphLayout=layout;
    }

    /** Sets the JGraph's facade
     * @param facade 
     */
    public void setVGraphFacade(JGraphFacade facade){
        if(facade!=null)
            vGraphFacade=facade;
    }

    /** Returns the Vertex's Color
     * @return 
     */
    public Color getVertexColor(){
        return vColor;
    }

    /** Returns the Edge's Color
     * @return 
     */
    public Color getEdgeColor(){
        return eColor;
    }

    /** Easier way to set the Characteristics of a VGraph(it's layout, edgeStyle, vertexColor and edgeColor)
     * @param vLayout 
     * @param edgeStyle
     * @param vColor
     * @param eColor  
     */
    public static void setCharacteristics(int vLayout, int edgeStyle, Color vColor, Color eColor){
        setVLayout(vLayout);
        setEdgeStyle(edgeStyle);
        setVertexColor(vColor);
        setEdgeColor(eColor);
    }

    /** Starts the "Loading..." Frame to notify the user about the delay*/
    private void startLoadingFrame(){
       frame = new JFrame("Loading");
       frame.setLayout(new BorderLayout(5,5));
       frame.setVisible(true);
       frame.getContentPane().add(new JLabel("Loading, this might take some minutes.. Please wait..."), BorderLayout.NORTH);
       progressBar.setSize(340, 50);
       frame.getContentPane().add(progressBar, BorderLayout.CENTER);
       frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
       frame.setBounds(200, 200, 340, 60);
       frame.setResizable(false);
       java.net.URL imgURL = getClass().getResource("images/graph.png");
       if (imgURL != null)
            frame.setIconImage(new ImageIcon(imgURL, "Graph").getImage());
    }

    /** Stops the "Loading..." Frame */
    private void stopLoadingFrame(){
        frame.setVisible(false);
        frame.dispose();
    }

}
